package com.flyfish.framework.beans.meta;

import com.flyfish.framework.beans.annotations.RestBean;
import com.flyfish.framework.domain.base.Domain;
import lombok.Getter;

import java.beans.PropertyDescriptor;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.function.Predicate;

/**
 * bean属性的类型
 * 包含java类型到js的映射
 * 我们通过类型自动解析bean的缺省控件，并自动映射
 * @author wangyu
 */
@Getter
public enum BeanPropertyType {

    STRING("String", CharSequence.class),
    NUMBER("Number", Number.class),
    BOOLEAN("Boolean", Boolean.class),
    DATE("Date", Date.class),
    ENUM("Enum", Enum.class),
    LIST("Array", Collection.class),
    DB_REF("Ref", descriptor -> {
        Class<?> clazz = descriptor.getPropertyType();
        return Domain.class.isAssignableFrom(clazz) && clazz.isAnnotationPresent(RestBean.class);
    }),
    OBJECT("Object", Object.class);

    private final Predicate<PropertyDescriptor> acceptor;

    BeanPropertyType(String name, Class<?>... classes) {
        this.name = name;
        this.classes = Arrays.asList(classes);
        this.acceptor = null;
    }

    private final String name;

    private final List<Class<?>> classes;

    BeanPropertyType(String name, Predicate<PropertyDescriptor> predicate) {
        this.name = name;
        this.classes = null;
        this.acceptor = predicate;
    }

    public static BeanPropertyType of(PropertyDescriptor descriptor) {
        for (BeanPropertyType type : values()) {
            if (null != type.classes && type.classes.stream().anyMatch(item -> item.isAssignableFrom(descriptor.getPropertyType())) ||
                    null != type.acceptor && type.acceptor.test(descriptor)) {
                return type;
            }
        }
        return null;
    }
}
